#!/usr/bin/env python
#
# Copyright 2015 Falldog Hsieh <falldog7@gmail.com>
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import os
import subprocess
from test import base

from src.config import PASSPHRASE_ENV


class TestWheel(base.TestPyConcreteBase):
    def test_building_by_wheel(self):
        env = os.environ.copy()
        env[PASSPHRASE_ENV] = 'test'
        sp = subprocess.run('pip wheel .', env=env, shell=True, stdout=subprocess.PIPE, stderr=subprocess.STDOUT)
        if sp.returncode != 0:
            print(sp.stdout.decode('utf-8'))
            raise Exception("pip create wheel fail...")



#!/usr/bin/env python
# -*- coding: utf8 -*-
# Create on : 2019/07/13
from __future__ import unicode_literals

print('HEAD')
from math import sqrt
import sys
print(sys.path)
print('HEAD2')
from joblib import Parallel, delayed
print('HEAD3')
p = Parallel(n_jobs=2, prefer='threads')(delayed(sqrt)(i ** 2) for i in range(10))
print('Hello')
print(p)
print('World')



#!/usr/bin/env python
# -*- coding: utf8 -*-
# Create on : 2019/07/13
from __future__ import unicode_literals

import os
from os.path import join
from test import base
from test.utility import ImportedTestCase, ImportedTestCaseError


class TestExe(base.TestPyConcreteBase):
    def discover(self):
        test_cases = []
        test_case_dir = join(base.ROOT_DIR, 'test', 'exe_testcases')
        for d in os.listdir(test_case_dir):
            test_case_path = join(test_case_dir, d)
            itc = ImportedTestCase(self._pyconcrete_exe, test_case_path)
            if itc.is_available_test_case():
                test_cases.append(itc)
            else:
                print('test_exe_testcases - {itc.module_name} (skip)'.format(itc=itc))
        return sorted(test_cases, key=lambda x: x.module_name)

    def test_auto_loading(self):
        test_cases = self.discover()
        error = False
        for tc in test_cases:
            print('test_exe_testcases - {tc.module_name} ... '.format(tc=tc), end='')
            try:
                res = tc.run()
                if res:
                    self.assertTrue(res, "{tc.module_name} validate failed".format(tc=tc))
                    print('Success')
                else:
                    print('Fail')
            except ImportedTestCaseError as e:
                print('Validate Exception')
                print('{{')
                print('  {tc.module_name} tmp_dir=`{tc.tmp_dir}`'.format(tc=tc))
                print('  return code = {return_code}'.format(return_code=e.return_code))
                print('  ======= output lines ======')
                print('  ' + '\n  '.join(e.output_lines))
                print('  ======= validate_errors ======')
                print('  ' + e.validate_errors)
                print('}}')
                error = True
            except Exception as e:
                print('Exception')
                print(str(e))
                print('{{')
                print('  {tc.module_name} tmp_dir=`{tc.tmp_dir}`'.format(tc=tc))
                print('}}')
                raise
            tc.close()

        assert error is False



#!/usr/bin/env python
# -*- coding: utf8 -*-
# Create on : 2019/07/13
from __future__ import unicode_literals
from multiprocessing import Pool


def sqrt(x):
    return x*x


if __name__ == '__main__':
    with Pool(5) as p:
        for r in p.map(sqrt, [1, 2, 3]):
            print(r)



#!/usr/bin/env python
# -*- coding: utf8 -*-
# Create on : 2019/07/13
from __future__ import unicode_literals

import imp
import os
import shutil
import subprocess
import sys
import tempfile
from os.path import abspath, basename, dirname, isdir, join

ROOT_DIR = abspath(join(dirname(__file__), '..'))


SOURCE_DIR_NAME = 'src'
MODULE_PREFIX = 'test_'

MAIN_PY = 'main.py'
MAIN_PYE = 'main.pye'
VALIDATOR_PY = 'validator.py'


class ImportedTestCaseError(Exception):
    def __init__(self, output_lines, return_code, validate_errors, *args, **kwargs):
        self.output_lines = output_lines
        self.return_code = return_code
        self.validate_errors = validate_errors


class ImportedTestCase(object):
    def __init__(self, pyconcrete_exe, module_path):
        self.pyconcrete_exe = pyconcrete_exe
        self.module_path = module_path
        self.tmp_dir = tempfile.mkdtemp(prefix='pyconcrete_testcase_')

    def close(self):
        shutil.rmtree(self.tmp_dir)

    @property
    def module_name(self):
        return basename(self.module_path)

    @property
    def module_src_path(self):
        return join(self.module_path, SOURCE_DIR_NAME)

    def is_available_test_case(self):
        if not isdir(self.module_path):
            return False
        if not self.module_name.startswith(MODULE_PREFIX):
            return False

        module_files = os.listdir(self.module_path)
        if VALIDATOR_PY not in module_files:
            return False
        if SOURCE_DIR_NAME not in module_files:
            return False

        module_code_files = os.listdir(self.module_src_path)
        if MAIN_PY not in module_code_files:
            return False

        return True

    def run(self):
        main_pye_path = self.build_pye()
        ret_data = self.execute(main_pye_path)
        return self.validate(ret_data, main_pye_path)

    def build_pye(self):
        dest_dir = join(self.tmp_dir, self.module_name)
        shutil.copytree(self.module_src_path, dest_dir)

        lib_compile_pye(dest_dir, remove_py=True, remove_pyc=True)
        return join(dest_dir, 'main.pye')

    def execute(self, main_pye_path):
        p = subprocess.Popen(
            [self.pyconcrete_exe, main_pye_path],
            stdin=subprocess.DEVNULL,
            stdout=subprocess.PIPE,
            stderr=subprocess.STDOUT,
        )
        output, error = p.communicate()
        output = output.decode('utf8')
        output = output.replace('\r\n', '\n')
        return {
            'output_lines': output.strip().split('\n'),
            'return_code': p.returncode,
        }

    def validate(self, ret_data, main_pye_path):
        validator_py = join(self.module_path, VALIDATOR_PY)
        validator = imp.load_source('', validator_py)
        try:
            return validator.validate(
                ret_data['output_lines'],
                main_pye_path=main_pye_path,
                return_code=ret_data['return_code'],
            )
        except Exception as e:
            raise ImportedTestCaseError(ret_data['output_lines'], ret_data['return_code'], str(e))


def lib_compile_pyc(folder, remove_py=False):
    from test.base import get_pyconcrete_env_path

    admin_path = join(ROOT_DIR, 'pyconcrete-admin.py')
    arg_remove_py = '--remove-py' if remove_py else ''
    subprocess.check_call(
        '%s %s compile --source=%s --pyc %s' % (sys.executable, admin_path, folder, arg_remove_py),
        env=get_pyconcrete_env_path(),
        shell=True,
    )


def lib_compile_pye(folder, remove_py=False, remove_pyc=False):
    from test.base import get_pyconcrete_env_path

    admin_path = join(ROOT_DIR, 'pyconcrete-admin.py')
    arg_remove_py = '--remove-py' if remove_py else ''
    arg_remove_pyc = '--remove-pyc' if remove_pyc else ''
    subprocess.check_call(
        '%s %s compile --source=%s --pye %s %s' % (sys.executable, admin_path, folder, arg_remove_py, arg_remove_pyc),
        env=get_pyconcrete_env_path(),
        shell=True,
    )



#!/usr/bin/env python
#
# Copyright 2015 Falldog Hsieh <falldog7@gmail.com>
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import logging
import os
import sys
import time
import unittest
from multiprocessing import Process, Queue
from os.path import abspath, dirname, join
from test import base
from zipfile import ZipFile

CUR_DIR = abspath(dirname(__file__))
ROOT_DIR = abspath(join(CUR_DIR, '..'))
DATA_DIR = join(CUR_DIR, 'data')
REQUEST_ZIP = join(DATA_DIR, 'requests-2.27.1.zip')  # zip structure: first level must contain `requests` folder
REQUEST_MAIN = join(DATA_DIR, 'main_requests.py')
PYADMIN_PATH = join(ROOT_DIR, 'pyconcrete-admin.py')
RUN_COUNT = int(os.environ.get('TEST_PYE_PERFORMANCE_COUNT', '5'))

logger = logging.getLogger('pyconcrete')


def main_requests(import_concrete, q):
    """
    testing main function for multiprocessing
    purpose: testing import without exception
    """
    if import_concrete:
        import pyconcrete  # noqa: F401

    t = time.time()
    import requests  # noqa: F401
    from requests.adapters import HTTPAdapter  # noqa: F401
    from requests.auth import HTTPDigestAuth, _basic_auth_str  # noqa: F401
    from requests.compat import Morsel, builtin_str, cookielib, getproxies, is_py3, str, urljoin, urlparse  # noqa: F401
    from requests.cookies import cookiejar_from_dict, morsel_to_cookie  # noqa: F401
    from requests.exceptions import ConnectionError  # noqa: F401
    from requests.exceptions import ConnectTimeout  # noqa: F401
    from requests.exceptions import InvalidSchema  # noqa: F401
    from requests.exceptions import InvalidURL  # noqa: F401
    from requests.exceptions import MissingSchema  # noqa: F401
    from requests.exceptions import ReadTimeout  # noqa: F401
    from requests.exceptions import RetryError  # noqa: F401
    from requests.exceptions import Timeout  # noqa: F401
    from requests.hooks import default_hooks  # noqa: F401
    from requests.models import PreparedRequest, urlencode  # noqa: F401
    from requests.sessions import SessionRedirectMixin  # noqa: F401
    from requests.structures import CaseInsensitiveDict  # noqa: F401

    t = time.time() - t
    q.put(requests.__file__)
    q.put(t)


@unittest.skipIf(not os.path.exists(REQUEST_ZIP), "requests zip file doesn't exists")
class TestPerformance(base.TestPyConcreteBase):
    def setUp(self):
        super(TestPerformance, self).setUp()

        zip = ZipFile(REQUEST_ZIP)
        zip.extractall(self.tmp_dir)
        zip.close()

        self.req_dir = join(self.tmp_dir, 'requests')
        base.touch(join(self.req_dir, '__init__.py'))

    def _test_requests(self, import_concrete):
        sys.path.insert(0, self.req_dir)

        q = Queue()
        p = Process(target=main_requests, args=(import_concrete, q))

        p.start()
        path = q.get(timeout=5)
        t = q.get(timeout=2)
        p.join()

        self.assertTrue(path.startswith(self.req_dir), "wrong import path of requests = %s" % path)
        return t

    def test_requests_pye(self):
        self.lib_compile_pye(self.req_dir, remove_py=True, remove_pyc=True)
        t = 0.0
        for i in range(RUN_COUNT):
            t += self._test_requests(True)
        logger.info(
            'test import request (pye) [count=%d] total time = %.2f, avg time = %.2f' % (RUN_COUNT, t, t / RUN_COUNT)
        )

    def test_requests_pyc(self):
        self.lib_compile_pyc(self.req_dir, remove_py=True)
        t = 0.0
        for i in range(RUN_COUNT):
            t += self._test_requests(False)
        logger.info(
            'test import request (pyc) [count=%d] total time = %.2f, avg time = %.2f' % (RUN_COUNT, t, t / RUN_COUNT)
        )

    def test_requests_pyc_with_import_hooker(self):
        self.lib_compile_pyc(self.req_dir, remove_py=True)
        t = 0.0
        for i in range(RUN_COUNT):
            t += self._test_requests(True)
        logger.info(
            'test import request (pyc) (import hooker) [count=%d] total time = %.2f, avg time = %.2f'
            % (RUN_COUNT, t, t / RUN_COUNT)
        )

    def test_requests_py(self):
        t = 0.0
        for i in range(RUN_COUNT):
            t += self._test_requests(False)
        logger.info(
            'test import request (py) [count=%d] total time = %.2f, avg time = %.2f' % (RUN_COUNT, t, t / RUN_COUNT)
        )

    def test_requests_py_with_import_hooker(self):
        t = 0.0
        for i in range(RUN_COUNT):
            t += self._test_requests(True)
        logger.info(
            'test import request (py) (import hooker) [count=%d] total time = %.2f, avg time = %.2f'
            % (RUN_COUNT, t, t / RUN_COUNT)
        )


if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python
#
# Copyright 2015 Falldog Hsieh <falldog7@gmail.com>
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import atexit
import hashlib
import imp
import os
import sys
import sysconfig
from distutils.command.build import build
from distutils.command.build_ext import build_ext
from distutils.command.clean import clean
from distutils.command.install import install
from distutils.core import Command, Extension, setup
from distutils.dist import Distribution
from os.path import join

from src.config import EXE_SRC_DIR, EXT_SRC_DIR, PASSPHRASE_ENV, PY_SRC_DIR, SECRET_HEADER_PATH, SRC_DIR, TEST_DIR

version_mod = imp.load_source('version', join(PY_SRC_DIR, 'version.py'))
version = version_mod.__version__
secret_key_header_created = False

PY2 = sys.version_info.major < 3


# .rst should created by pyconcrete-admin
if os.path.exists('README.rst'):
    readme_path = 'README.rst'
else:
    readme_path = 'README.md'
with open(readme_path, 'r') as f:
    readme = f.read()

try:
    input = raw_input
except NameError:
    pass


def is_mingw():
    """Compiler is mingw."""
    dst = Distribution()
    dst.parse_config_files()
    if 'build' not in dst.command_options:
        return False
    _build = dst.command_options['build']
    if 'compiler' not in _build:
        return False
    compiler = _build['compiler']
    if compiler[1].startswith('mingw'):
        return True
    return False


def is_msvc():
    return sys.platform == 'win32' and not is_mingw()


def is_mac():
    return sys.platform == 'darwin'


def hash_key(key):
    if PY2:
        factor = sum([ord(s) for s in key])
    else:
        factor = sum([s for s in key])
    factor %= 128
    if factor < 16:
        factor += 16

    m = hashlib.md5()
    m.update(key)
    k = m.digest()

    return k, factor


def create_secret_key_header(key, factor):
    # reference from - http://stackoverflow.com/questions/1356896/how-to-hide-a-string-in-binary-code
    # encrypt the secret key in binary code
    # avoid to easy read from HEX view
    global secret_key_header_created
    if secret_key_header_created:
        return

    key_val_lst = []
    for i, k in enumerate(key):
        n = ord(k) if PY2 else k
        key_val_lst.append("(0x%X ^ (0x%X - %d))" % (n, factor, i))
    key_val_code = ", ".join(key_val_lst)

    code = """
        #define SECRET_NUM 0x%X
        #define SECRET_KEY_LEN %d
        static const unsigned char* GetSecretKey()
        {
            unsigned int i = 0;
            static unsigned char key[] = {%s, 0/* terminal char */};
            static int is_encrypt = 1/*true*/;
            if( is_encrypt )
            {
                for(i = 0 ; i < SECRET_KEY_LEN ; ++i)
                {
                    key[i] = key[i] ^ (SECRET_NUM - i);
                }
                is_encrypt = 0/*false*/;
            }
            return key;
        }
    """ % (
        factor,
        len(key),
        key_val_code,
    )

    with open(SECRET_HEADER_PATH, 'w') as f:
        f.write(code)

    secret_key_header_created = True
    atexit.register(remove_secret_key_header)


def remove_secret_key_header():
    if os.path.exists(SECRET_HEADER_PATH):
        os.remove(SECRET_HEADER_PATH)


class ExeDistribution(Distribution):
    exe_modules = ()

    def has_exe_modules(self):
        return self.exe_modules and len(self.exe_modules) > 0


# ================================================= command ================================================= #


class CmdBase:
    def pre_process(self):
        if secret_key_header_created:
            return

        if not self.passphrase:
            self.passphrase = os.getenv(PASSPHRASE_ENV)

        if not self.passphrase:
            self.passphrase = input(
                "please input the passphrase \nfor encrypt your python script (enter for default) : \n"
            )
            if len(self.passphrase) == 0:
                raise Exception("empty passphrase is not allow")
            else:
                passphrase2 = input("please input again to confirm\n")
                if self.passphrase != passphrase2:
                    raise Exception("Passphrase is different")

        k, f = hash_key(self.passphrase.encode('utf8'))
        create_secret_key_header(k, f)

    def post_process(self):
        pass


class BuildEx(CmdBase, build):
    """
    execute extra function before/after build.run()
    """

    user_options = build.user_options + [('passphrase=', None, 'specify passphrase')]

    def initialize_options(self):
        build.initialize_options(self)
        self.passphrase = None

    def run(self):
        self.pre_process()
        ret = build.run(self)
        self.post_process()
        return ret


class BuildExe(CmdBase, build_ext):
    """
    execute extra function before/after build.run()
    """

    user_options = build.user_options + [('passphrase=', None, 'specify passphrase')]

    def initialize_options(self):
        build_ext.initialize_options(self)
        self.passphrase = None
        self.build_scripts = None

    def finalize_options(self):
        build_ext.finalize_options(self)
        self.set_undefined_options('build', ('build_scripts', 'build_scripts'))
        self.extensions = self.distribution.exe_modules

    def run(self):
        self.pre_process()
        build_ext.run(self)
        self.post_process()

    def build_extensions(self):
        """
        copy from distutils/command/build_ext.py build_ext.build_extensions
        just override build_extensions for covert the compiler setup from build_ext.run()
        """
        ext = self.extensions[0]  # only one exe extension

        macros = ext.define_macros[:]
        for undef in ext.undef_macros:
            macros.append((undef,))

        objects = self.compiler.compile(
            list(ext.sources),
            output_dir=self.build_temp,
            include_dirs=ext.include_dirs,
            macros=macros,
            debug=self.debug,
            depends=ext.depends,
        )

        self._built_objects = objects[:]

        # link for executable file
        self.compiler.link_executable(
            objects,
            ext.name,
            libraries=self.get_libraries(ext),
            library_dirs=ext.library_dirs,
            runtime_library_dirs=ext.runtime_library_dirs,
            extra_postargs=ext.extra_link_args,
            output_dir=self.build_scripts,  # should not use build_lib, it will be copied to python site-packages
            debug=self.debug,
        )


class InstallEx(CmdBase, install):
    """
    execute extra function before/after install.run()
    """

    user_options = install.user_options + [('passphrase=', None, 'specify passphrase')]
    sub_commands = install.sub_commands + [('build_exe', lambda self: True)]

    def initialize_options(self):
        install.initialize_options(self)
        self.passphrase = None
        self.build_scripts = None

    def finalize_options(self):
        install.finalize_options(self)
        self.set_undefined_options('build', ('build_scripts', 'build_scripts'))

    def _clean_build(self):
        # for clear old build cached (majorly for *.o)
        # but, `pip install pyconcrete` will build wheel first, then do installation, should skip the case avoid
        # to clear current building files
        for cmd, ran in self.distribution.have_run.items():
            if cmd.startswith('build') and ran:
                return

        c = clean(self.distribution)
        c.all = True
        c.finalize_options()
        c.run()

    def run(self):
        self._clean_build()

        self.pre_process()
        ret = install.run(self)
        self.install_exe()
        self.post_process()
        self.create_pth()
        return ret

    def create_pth(self):
        """
        create pyconcrete.pth for automatic import pyconcrete after python launched
        """
        # setup(
        #     data_files=(sysconfig.get_python_lib(), 'pyconcrete.pth'),
        #     ...
        # )
        # it will got wrong result in ubuntu, get_python_lib got /usr/lib
        # but pyconcrete installed in /usr/local/lib
        #
        # get install_libbase to avoid this issue
        filename = join(self.install_libbase, 'pyconcrete.pth')
        with open(filename, 'w') as f:
            f.write('import pyconcrete')
        print('creating %s' % filename)

    def install_exe(self):
        if sys.platform == 'win32':
            # install `pyconcrete.exe` to %PYTHON%/Scripts
            exe_name = 'pyconcrete.exe'
        else:
            # install `pyconcrete` to /usr/local/bin
            exe_name = 'pyconcrete'
        self.copy_file(os.path.join(self.build_scripts, exe_name), os.path.join(self.install_scripts, exe_name))


# ================================================= test command ================================================= #


class TestEx(Command):
    description = "Running all unit test for pyconcrete"
    user_options = []

    def initialize_options(self):
        pass

    def finalize_options(self):
        pass

    def run(self):
        import unittest

        suite = unittest.TestLoader().discover(TEST_DIR)
        unittest.TextTestRunner(verbosity=2).run(suite)


# ================================================= extension ================================================= #


def get_include_dirs():
    openaes_include_dirs = [
        join(EXT_SRC_DIR),
        join(EXT_SRC_DIR, 'openaes', 'inc'),
    ]
    if is_msvc() and PY2:  # Only Python 2.7 & 3.2 Need VisualStudio 2008 (without stdint.h)
        openaes_include_dirs.append(join(EXT_SRC_DIR, 'include_win'))
    return openaes_include_dirs


def get_libraries(include_python_lib=False):
    libraries = []
    if sys.version_info.major == 3 and sys.version_info.minor >= 5 and is_msvc():
        # https://stackoverflow.com/questions/32418766/c-unresolved-external-symbol-sprintf-and-sscanf-in-visual-studio-2015
        libraries = ['legacy_stdio_definitions']

    if is_msvc():
        link_py = 'python{0}{1}'.format(sys.version_info.major, sys.version_info.minor)
    else:
        # PEP 3149 -- ABI version tagged .so files
        # link python lib may be: `-l python3.3` or `-l python3.6m`
        # reference link: https://www.python.org/dev/peps/pep-3149/
        link_py_fmt = 'python{version}{abiflags}'
        abiflags = getattr(sys, 'abiflags', '')
        link_py = link_py_fmt.format(
            version=sysconfig.get_python_version(),
            abiflags=abiflags,
        )

    if include_python_lib:
        libraries.append(link_py)

    return libraries


def get_exe_link_args():
    ver = '%d.%d' % (sys.version_info.major, sys.version_info.minor)
    if is_msvc() and (ver == '3.3' or ver == '3.4'):
        # For Fix Manifest error, https://bugs.python.org/issue4431
        return ['/MANIFEST']

    # add -Lxxxx for link correct lib -lpython3.x
    if is_mac():
        if not sysconfig.get_config_var('Py_ENABLE_SHARED'):  # reference from `python-config --ldflags`
            return ['-L' + sysconfig.get_config_var('LIBPL')]

    return None


include_dirs = get_include_dirs()
openaes_sources = [
    join(EXT_SRC_DIR, 'openaes', 'src', 'oaes_base64.c'),
    join(EXT_SRC_DIR, 'openaes', 'src', 'oaes_lib.c'),
]

ext_module = Extension(
    'pyconcrete._pyconcrete',
    include_dirs=include_dirs,
    libraries=get_libraries(include_python_lib=False),
    sources=[
        join(EXT_SRC_DIR, 'pyconcrete.c'),
        join(EXT_SRC_DIR, 'pyconcrete_module.c'),
    ]
    + openaes_sources,
)

exe_module = Extension(
    'pyconcrete',
    include_dirs=include_dirs,
    libraries=get_libraries(include_python_lib=True),
    define_macros=[('PYCONCRETE_VERSION', '"%s"' % version)],
    extra_link_args=get_exe_link_args(),
    sources=[
        join(EXE_SRC_DIR, 'pyconcrete_exe.c'),
        join(EXT_SRC_DIR, 'pyconcrete.c'),
    ]
    + openaes_sources,
)

# ================================================= setup ================================================= #


setup(
    name='pyconcrete',
    version=version,
    description='Protect your python script, encrypt it as .pye and decrypt when import it',
    long_description=readme,
    keywords='python source encryption obfuscation',
    author='Falldog',
    author_email='falldog7@gmail.com',
    url='https://github.com/Falldog/pyconcrete',
    license="Apache License 2.0",
    ext_modules=[ext_module],
    exe_modules=[exe_module],
    cmdclass={
        "build": BuildEx,
        "build_exe": BuildExe,
        "install": InstallEx,
        "test": TestEx,
    },
    scripts=[
        'pyconcrete-admin.py',
    ],
    packages=[
        'pyconcrete',
    ],
    package_dir={
        '': SRC_DIR,
    },
    classifiers=[
        'Development Status :: 4 - Beta',
        'Intended Audience :: Developers',
        'Intended Audience :: System Administrators',
        'Topic :: Software Development :: Build Tools',
        'Topic :: Security',
        'Topic :: Security :: Cryptography',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9',
        'Programming Language :: Python :: 3.10',
        'Programming Language :: Python :: Implementation :: CPython',
        'License :: OSI Approved :: Apache Software License',
    ],
    distclass=ExeDistribution,
)
